@@ -157,6 +157,12 @@ if test "x$fvm_gcc" = "xgcc"; then
 
   esac
 
+  case "$host_os" in
+    *cygwin)
+    cflags_default="`echo $cflags_default | sed -e 's/c99/gnu99/g'`"
+    ;;
+  esac
+
 # Otherwise, are we using icc ?
 #------------------------------
 
@@ -185,10 +191,12 @@ elif test "x$fvm_gcc" = "xicc"; then
       ;;
   esac
 
+fi
+
 # Otherwise, are we using pgcc ?
 #-------------------------------
 
-else
+if test "x$fvm_compiler_known" != "xyes" ; then
 
   $CC -V 2>&1 | grep 'The Portland Group' > /dev/null
   if test "$?" = "0" ; then
@@ -202,7 +210,7 @@ else
 
     # Default compiler flags
     cflags_default="-Xa -fPIC"
-    cflags_default_dbg="-g"
+    cflags_default_dbg="-g -Mbounds"
     cflags_default_opt="-fast -fastsse"
     cflags_default_prf="-Mprof=func,lines"
 
@@ -210,6 +218,64 @@ else
 
 fi
 
+# Otherwise, are we using xlc ?
+#------------------------------
+
+if test "x$fvm_compiler_known" != "xyes" ; then
+
+  $CC -qversion 2>&1 | grep 'XL C' > /dev/null
+  if test "$?" = "0" ; then
+
+    echo "compiler '$CC' is IBM XL C compiler"
+
+    # Version strings for logging purposes and known compiler flag
+    $CC -qversion > $outfile 2>&1
+    fvm_ac_cc_version=`grep 'XL C' $outfile`
+    fvm_compiler_known=yes
+    fvm_linker_set=yes
+
+    # Default compiler flags
+    cflags_default="-q64"
+    cflags_default_opt="-O3"
+    cflags_default_hot="-O3"
+    cflags_default_dbg="-g"
+    cflags_default_prf="-pg"
+
+    # Default  linker flags
+    ldflags_default=""
+    ldflags_default_opt="-g -O3"
+    ldflags_default_dbg="-g"
+    ldflags_default_prf="-pg"
+
+    # Disable shared libraries in all cases
+    fvm_disable_shared=yes
+
+    # Adjust options for IBM Blue Gene cross-compiler
+
+    grep 'Blue Gene' $outfile > /dev/null
+    if test "$?" = "0" ; then
+      # Default compiler flags
+      fvm_ibm_bg_type=`grep 'Blue Gene' $outfile | sed -e 's/.*Blue Gene\/\([A-Z]\).*/\1/'`
+      if test "$fvm_ibm_bg_type" = "L" ; then
+        cppflags_default="-I/bgl/BlueLight/ppcfloor/bglsys/include"
+        cflags_default="-g -qmaxmem=-1 -qarch=440d -qtune=440"
+        cflags_default_opt="-O3"
+        cflags_default_hot="-O3 -qhot"
+        cflags_default_dbg=""
+        ldflags_default="-Wl,-allow-multiple-definition -L/bgl/BlueLight/ppcfloor/bglsys/lib -lmpich.rts -lmsglayer.rts -lrts.rts -ldevices.rts -lnss_files -lnss_dns -lresolv"
+      elif test "$fvm_ibm_bg_type" = "P" ; then
+        cppflags_default="-I/bgsys/drivers/ppcfloor/comm/include"
+        cflags_default="-g -qmaxmem=-1 -qarch=450d -qtune=450"
+        cflags_default_opt="-O3"
+        cflags_default_hot="-O3 -qhot"
+        cflags_default_dbg=""
+        ldflags_default="-Wl,-allow-multiple-definition -L/bgsys/drivers/ppcfloor/comm/lib -lmpich.cnk -ldcmfcoll.cnk -ldcmf.cnk"
+      fi
+    fi
+
+  fi
+fi
+
 # Compiler still not identified
 #------------------------------
 
@@ -217,53 +283,6 @@ if test "x$fvm_compiler_known" != "xyes" ; then
 
   case "$host_os" in
 
-    linux* | none)
-
-      # IBM Blue Gene
-      #--------------
-
-      $CC -qversion 2>&1 | grep 'XL C' | grep 'Blue Gene' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$CC' is IBM XL C compiler for Blue Gene"
-
-        # Version strings for logging purposes and known compiler flag
-        $CC -qversion > $outfile 2>&1
-        fvm_ac_cc_version=`grep 'XL C' $outfile`
-        fvm_compiler_known=yes
-
-        # Default compiler flags
-        if test -d /bgl/BlueLight/ppcfloor/bglsys/include ; then
-          cppflags_default="-I/bgl/BlueLight/ppcfloor/bglsys/include"
-          cflags_default="-qmaxmem=-1 -qarch=440d -qtune=440"
-        elif test -d /bgsys/drivers/ppcfloor/comm/include ; then
-          cppflags_default="-I/bgsys/drivers/ppcfloor/comm/include"
-          cflags_default="-qmaxmem=-1 -qarch=450d -qtune=450"
-        else
-          cppflags_default=""
-        fi
-        cflags_default_opt="-g -O3"
-        cflags_default_prf="-pg"
-
-        # Default linker flags
-        if test -d /bgl/BlueLight/ppcfloor/bglsys ; then
-          ldflags_default="-Wl,-allow-multiple-definition -L/bgl/BlueLight/ppcfloor/bglsys/lib -lmpich.rts -lmsglayer.rts -lrts.rts -ldevices.rts -lnss_files -lnss_dns -lresolv"
-        elif test -d /bgsys/drivers/ppcfloor/comm/include ; then
-          ldflags_default="-Wl,-allow-multiple-definition -L/bgsys/drivers/ppcfloor/comm/lib -lmpich.cnk -ldcmfcoll.cnk -ldcmf.cnk"
-        else
-          ldflags_default=""
-        fi
-        ldflags_default_opt="-g -O3"
-        ldflags_default_dbg="-g"
-        ldflags_default_prf="-pg"
-        fvm_linker_set=yes
-
-        # Disable shared libraries in all cases
-        fvm_disable_shared=yes
-
-      fi
-      ;;
-
     osf*)
 
       # Native Compaq Tru64 Unix C compiler
@@ -289,18 +308,23 @@ if test "x$fvm_compiler_known" != "xyes" ; then
           ;;
         esac
 
+        # Default  linker flags
+        ldflags_default="-Wl,-call_shared"
+        ldflags_default_opt="-O0"
+        ldflags_default_dbg="-g"
+        ldflags_default_prf="-pg"
       fi
       ;;
 
-    uxpv*)
+    SUPER-UX* | superux*)
 
-      # Native Fujitsu vectorizing C compiler (tested on VPP5000)
-      #---------------------------------------
+      # Native NEC SX vectorizing C compiler (sxmpicc)
+      #-------------------------------------
 
-      $CC -V 2>&1 | grep 'Fujitsu' > /dev/null
+      $CC -V conftest.c 2>&1 | grep 'NEC' | grep 'SX' > /dev/null
       if test "$?" = "0" ; then
 
-        echo "compiler '$CC' is Fujitsu compiler"
+        echo "compiler '$CC' is NEC SX compiler"
 
         # Version strings for logging purposes and known compiler flag
         $CC -V conftest.c > $outfile 2>&1
@@ -309,17 +333,16 @@ if test "x$fvm_compiler_known" != "xyes" ; then
         fvm_linker_set=yes
 
         # Default compiler flags
-        cflags_default="-KA64 -Kvp"
-        cflags_default_opt="-O"
-        cflags_default_dbg="-g -Kargchk -w4"
-        cflags_default_prf="-pg"
+        cflags_default="-Kc99 -pvctl,loopcnt=2147483647"
+        cflags_default_opt=""
+        cflags_default_dbg=""
+        cflags_default_prf=""
 
         # Default linker flags
-        ldflags_default="-Kvp -Kargchk -Wl,-S1:d"
+        ldflags_default=""
         ldflags_default_opt="-O"
         ldflags_default_dbg="-g"
         ldflags_default_prf="-pg"
-        fvm_linker_set=yes
 
       fi
       ;;
