@@ -162,10 +162,10 @@ _compute_displ(int        n_ranks,
  *----------------------------------------------------------------------------*/
 
 static void
-_ordered_list(size_t        n_ents,
-              fvm_gnum_t    ent_global_num[],
-              size_t       *n_ordered_ents,
-              fvm_lnum_t  **ordered_ent)
+_ordered_list(size_t              n_ents,
+              const fvm_gnum_t    ent_global_num[],
+              size_t             *n_ordered_ents,
+              fvm_lnum_t        **ordered_ent)
 {
   size_t i, j;
   size_t _n_ordered_ents = 1;
@@ -354,6 +354,7 @@ fvm_block_to_part_compute_sizes(int         rank_id,
 
   bi.gnum_range[0] = _bi.gnum_range[0];
   bi.gnum_range[1] = _bi.gnum_range[1];
+  bi.n_ranks = _bi.n_ranks;
   bi.rank_step = _bi.rank_step;
   bi.block_size = _bi.block_size;
 
@@ -654,7 +655,7 @@ fvm_block_to_part_t *
 fvm_block_to_part_create_adj(MPI_Comm                  comm,
                              fvm_block_to_part_info_t  adjacent_block,
                              size_t                    adjacency_size,
-                             fvm_gnum_t                adjacency[])
+                             const fvm_gnum_t          adjacency[])
 {
   int i;
   size_t j;
@@ -763,6 +764,118 @@ fvm_block_to_part_create_adj(MPI_Comm                  comm,
   return d;
 }
 
+/*----------------------------------------------------------------------------
+ * Initialize block to partition distributor based global element numbers
+ * for partitioned data.
+ *
+ * This function is similar to fvm_block_to_part_create_adj(), but is
+ * intended to be used with already partitioned data, which may have
+ * been renumbered; as such, it does not sort or remove duplicates from the
+ * list of global elements, and global entity numbers must be nonzero.
+ *
+ * arguments:
+ *   comm           <-- communicator
+ *   bi             <-- block size and range info
+ *   n_part_ents    <-- number of elements in partition
+ *   global_ent_num <-- global entity numbers (in partition)
+ *
+ * returns:
+ *   initialized partition to block distributor
+ *----------------------------------------------------------------------------*/
+
+fvm_block_to_part_t *
+fvm_block_to_part_create_by_gnum(MPI_Comm                   comm,
+                                 fvm_block_to_part_info_t   bi,
+                                 fvm_lnum_t                 n_part_ents,
+                                 const fvm_gnum_t           global_ent_num[])
+{
+  int i;
+  size_t j;
+
+  size_t recv_size = 0;
+  fvm_gnum_t *send_num = NULL, *recv_num = NULL;
+
+  fvm_block_to_part_t *d = _block_to_part_create(comm);
+
+  const int n_ranks = d->n_ranks;
+
+  d->n_part_ents = n_part_ents;
+
+  /* Use adjacent global number to determine to which ranks they will
+     be sent. Send and receive counts are exchanged here, as ranks
+     requesting (i.e receiving entities) are determined first */
+
+  for (i = 0; i < d->n_ranks; i++)
+    d->recv_count[i] = 0;
+
+  for (j = 0; j < d->n_part_ents; j++) {
+    fvm_gnum_t ent_g_id = global_ent_num[j] - 1;
+    int ent_ent_rank =   (ent_g_id / bi.block_size)
+                       * bi.rank_step;
+    d->recv_count[ent_ent_rank] += 1;
+  }
+
+  MPI_Alltoall(d->recv_count, 1, MPI_INT, d->send_count, 1, MPI_INT, comm);
+
+  d->send_size = _compute_displ(n_ranks, d->send_count, d->send_displ);
+  recv_size = _compute_displ(n_ranks, d->recv_count, d->recv_displ);
+
+  if (d->n_part_ents != recv_size)
+    bft_error
+      (__FILE__, __LINE__, 0,
+       _("inconsistent sizes computed for a block to partition distributor\n"
+         "(%lu expected, %lu determined)."),
+       (unsigned long)(d->n_part_ents), (unsigned long)recv_size);
+
+  /* Allocate distributor arrays */
+
+  BFT_MALLOC(d->send_list, d->send_size, fvm_lnum_t);
+  BFT_MALLOC(d->recv_order, d->n_part_ents, fvm_lnum_t);
+
+  BFT_MALLOC(d->_recv_global_num, d->n_part_ents, fvm_gnum_t);
+  d->recv_global_num = d->_recv_global_num;
+
+  /* We already have all the necessary info to build the global numbering */
+
+  for (j = 0; j < d->n_part_ents; j++)
+    d->_recv_global_num[j] = global_ent_num[j];
+
+  /* Prepare destination rank request and receive_order at the same time
+     (temporarily modifying d->recv_displ) */
+
+  BFT_MALLOC(send_num, d->send_size, fvm_gnum_t);
+  BFT_MALLOC(recv_num, d->n_part_ents, fvm_gnum_t);
+
+  for (j = 0; j < d->n_part_ents; j++) {
+    fvm_gnum_t ent_g_num = global_ent_num[j];
+    int ent_ent_rank =   ((ent_g_num-1) / bi.block_size)
+                       * bi.rank_step;
+    recv_num[d->recv_displ[ent_ent_rank]] = ent_g_num;
+    d->recv_order[j] = d->recv_displ[ent_ent_rank];
+    d->recv_displ[ent_ent_rank] += 1;
+  }
+
+  for (i = 0; i < n_ranks; i++)
+    d->recv_displ[i] -= d->recv_count[i];
+
+  MPI_Alltoallv(recv_num, d->recv_count, d->recv_displ, FVM_MPI_GNUM,
+                send_num, d->send_count, d->send_displ, FVM_MPI_GNUM,
+                d->comm);
+
+  BFT_FREE(recv_num);
+
+  /* Now prepare send list */
+
+  for (j = 0; j < d->send_size; j++)
+    d->send_list[j] = send_num[j] - bi.gnum_range[0];
+
+  BFT_FREE(send_num);
+
+  /* Return initialized structure */
+
+  return d;
+}
+
 /*----------------------------------------------------------------------------
  * Destroy a block to partition distributor structure.
  *
