@@ -9,7 +9,7 @@
   This file is part of the "Finite Volume Mesh" library, intended to provide
   finite volume mesh and associated fields I/O and manipulation services.
 
-  Copyright (C) 2008  EDF
+  Copyright (C) 2008-2009  EDF
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,14 @@
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
 
+/*----------------------------------------------------------------------------*/
+
+#include "fvm_config.h"
+
+#if defined(FVM_HAVE_MPI)
+#include <mpi.h>
+#endif
+
 /*----------------------------------------------------------------------------
  *  Local headers
  *----------------------------------------------------------------------------*/
@@ -55,6 +63,7 @@ typedef struct {
 
   fvm_gnum_t   gnum_range[2];  /* Start and past-the-end global numbers
                                   associated with local block */
+  int          n_ranks;        /* Number of active ranks */
   int          rank_step;      /* Step between active block ranks
                                   (1 in basic case, > 1 if we seek to
                                   avoid too small buffers and agglomerate
@@ -165,7 +174,27 @@ fvm_block_to_part_t *
 fvm_block_to_part_create_adj(MPI_Comm                  comm,
                              fvm_block_to_part_info_t  adj_bi,
                              size_t                    adjacency_size,
-                             fvm_gnum_t                adjacency[]);
+                             const fvm_gnum_t          adjacency[]);
+
+/*----------------------------------------------------------------------------
+ * Initialize block to partition distributor based global element numbers
+ * for partitioned data.
+ *
+ * arguments:
+ *   comm           <-- communicator
+ *   bi             <-- block size and range info
+ *   n_ents         <-- number of elements in partition
+ *   global_ent_num <-- global entity numbers (in partition)
+ *
+ * returns:
+ *   initialized partition to block distributor
+ *----------------------------------------------------------------------------*/
+
+fvm_block_to_part_t *
+fvm_block_to_part_create_by_gnum(MPI_Comm                   comm,
+                                 fvm_block_to_part_info_t   bi,
+                                 fvm_lnum_t                 n_ents,
+                                 const fvm_gnum_t           global_ent_num[]);
 
 /*----------------------------------------------------------------------------
  * Destroy a block to partition distributor structure.
