@@ -1589,23 +1589,23 @@ fvm_file_dump(const fvm_file_t  *f)
                              "FVM_FILE_MODE_APPEND"};
 
   if (f == NULL) {
-    bft_printf(_("\n"
-                 "Null file dump:\n"));
+    bft_printf("\n"
+               "Null file dump:\n");
     return;
   }
 
-  bft_printf(_("\n"
-               "File name:                \"%s\"\n"
-               "Access mode:              %s\n"
-               "Semantics:\n"
-               "  no_mpi_io:              %d\n"
-               "  no_predistribute:       %d\n"
-               "  explicit_offsets:       %d\n"
-               "  individual_pointers:    %d\n"
-               "Rank:                     %d\n"
-               "N ranks:                  %d\n"
-               "Swap endian:              %d\n"
-               "Serial handle:            %p\n"),
+  bft_printf("\n"
+             "File name:                \"%s\"\n"
+             "Access mode:              %s\n"
+             "Semantics:\n"
+             "  no_mpi_io:              %d\n"
+             "  no_predistribute:       %d\n"
+             "  explicit_offsets:       %d\n"
+             "  individual_pointers:    %d\n"
+             "Rank:                     %d\n"
+             "N ranks:                  %d\n"
+             "Swap endian:              %d\n"
+             "Serial handle:            %p\n",
              f->name, mode_name[f->mode],
              (f->semantics & FVM_FILE_NO_MPI_IO),
              (f->semantics & FVM_FILE_NO_PREDISTRIBUTE) >> 1,
@@ -1615,11 +1615,11 @@ fvm_file_dump(const fvm_file_t  *f)
              f->sh);
 
 #if defined(FVM_HAVE_MPI)
-  bft_printf(_("Associated communicator:  %lu\n"),
+  bft_printf("Associated communicator:  %lu\n",
              (unsigned long)(f->comm));
 #if defined(FVM_HAVE_MPI_IO)
-  bft_printf(_("MPI file handle:          %lu\n"
-               "MPI file offset:          %lu\n"),
+  bft_printf("MPI file handle:          %lu\n"
+               "MPI file offset:          %lu\n",
              (unsigned long)(f->fh),
              (unsigned long)(f->offset));
 #endif
