@@ -119,11 +119,6 @@ struct _fvm_gather_slice_t {
 
   _Bool  use_next_global_num;
 
-  /* Use "safe" communication scheme with extra synchronization,
-     avoiding calls to MPI_Probe() ? */
-
-  _Bool  safe_mode;
-
   /* Buffers associated with MPI Datatype creation */
 
   size_t       recv_buf_size;  /* Receive buffer size, in bytes */
@@ -342,8 +337,6 @@ fvm_gather_slice_create(const fvm_io_num_t  *entity_io_num,
 
   this_slice->use_next_global_num = false;
 
-  this_slice->safe_mode = fvm_parall_get_safe_gather_mode();
-
   /* Allocated buffers (blocklengths allocated only if needed for ranks > 0) */
 
   this_slice->recv_buf_size = 0;
@@ -624,16 +617,9 @@ fvm_gather_slice_index(const fvm_lnum_t     local_index[],
       if (   this_slice->next_global_num[distant_rank] < global_num_end
           || this_slice->use_next_global_num == false) {
 
-        if (this_slice->safe_mode == true) {
-          MPI_Send(&distant_rank, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm);
-          MPI_Recv(&n_distant_entities, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm, &status);
-        }
-        else {
-          MPI_Probe(distant_rank, FVM_MPI_TAG, comm, &status);
-          MPI_Get_count(&status, FVM_MPI_GNUM, &n_distant_entities);
-        }
+        MPI_Send(&distant_rank, 1, MPI_INT, distant_rank, FVM_MPI_TAG, comm);
+        MPI_Recv(&n_distant_entities, 1, MPI_INT,
+                 distant_rank, FVM_MPI_TAG, comm, &status);
 
         MPI_Recv(displacements, n_distant_entities, FVM_MPI_GNUM,
                  distant_rank, FVM_MPI_TAG, comm, &status);
@@ -685,14 +671,12 @@ fvm_gather_slice_index(const fvm_lnum_t     local_index[],
 
       /* Send local index to rank zero */
 
-      if (this_slice->safe_mode == true) {
-        int buf_val;
-        MPI_Recv(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm, &status);
-        buf_val = n_local_entities + 1;
-        MPI_Send(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm);
-      }
+      int buf_val;
+
+      MPI_Recv(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm, &status);
+
+      buf_val = n_local_entities + 1;
+      MPI_Send(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm);
 
       MPI_Send(displacements, n_local_entities + 1,
                FVM_MPI_GNUM, 0, FVM_MPI_TAG, comm);
@@ -938,16 +922,9 @@ fvm_gather_array(const void          *local_array,
       if (   this_slice->next_global_num[distant_rank] < global_num_end
           || this_slice->use_next_global_num == false) {
 
-        if (this_slice->safe_mode == true) {
-          MPI_Send(&distant_rank, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm);
-          MPI_Recv(&n_distant_entities, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm, &status);
-        }
-        else {
-          MPI_Probe(distant_rank, FVM_MPI_TAG, comm, &status);
-          MPI_Get_count(&status, FVM_MPI_GNUM, &n_distant_entities);
-        }
+        MPI_Send(&distant_rank, 1, MPI_INT, distant_rank, FVM_MPI_TAG, comm);
+        MPI_Recv(&n_distant_entities, 1, MPI_INT,
+                 distant_rank, FVM_MPI_TAG, comm, &status);
 
         MPI_Recv(displacements, n_distant_entities, FVM_MPI_GNUM,
                  distant_rank, FVM_MPI_TAG, comm, &status);
@@ -989,14 +966,12 @@ fvm_gather_array(const void          *local_array,
 
       /* Send local index to rank zero */
 
-      if (this_slice->safe_mode == true) {
-        int buf_val;
-        MPI_Recv(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm, &status);
-        buf_val = n_local_entities + 1;
-        MPI_Send(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm);
-      }
+      int buf_val;
+
+      MPI_Recv(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm, &status);
+
+      buf_val = n_local_entities + 1;
+      MPI_Send(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm);
 
       MPI_Send(displacements, n_local_entities + 1, FVM_MPI_GNUM, 0,
                FVM_MPI_TAG, comm);
@@ -1168,18 +1143,11 @@ fvm_gather_indexed(const void          *local_array,
       if (   this_slice->next_global_num[distant_rank] < global_num_end
           || this_slice->use_next_global_num == false) {
 
-        if (this_slice->safe_mode == true) {
-          MPI_Send(&distant_rank, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm);
-          MPI_Recv(&n_distant_entities, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm, &status);
-        }
-        else {
-          MPI_Probe(distant_rank, FVM_MPI_TAG, comm, &status);
-          MPI_Get_count(&status, FVM_MPI_GNUM, &n_distant_entities);
-        }
+        MPI_Send(&distant_rank, 1, MPI_INT, distant_rank, FVM_MPI_TAG, comm);
+        MPI_Recv(&n_distant_entities, 1, MPI_INT,
+                 distant_rank, FVM_MPI_TAG, comm, &status);
 
-      /* Get index from distant rank */
+        /* Get index from distant rank */
 
         MPI_Recv(displacements, n_distant_entities, FVM_MPI_GNUM,
                  distant_rank, FVM_MPI_TAG, comm, &status);
@@ -1230,14 +1198,12 @@ fvm_gather_indexed(const void          *local_array,
 
       /* Send local index to rank zero */
 
-      if (this_slice->safe_mode == true) {
-        int buf_val;
-        MPI_Recv(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm, &status);
-        buf_val = n_local_entities + 1;
-        MPI_Send(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm);
-      }
+      int buf_val;
+
+      MPI_Recv(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm, &status);
+
+      buf_val = n_local_entities + 1;
+      MPI_Send(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm);
 
       MPI_Send(displacements, n_local_entities + 1, FVM_MPI_GNUM,
                0, FVM_MPI_TAG, comm);
@@ -1380,16 +1346,9 @@ fvm_gather_strided_connect(const fvm_lnum_t    local_connect[],
       if (   this_slice->next_global_num[distant_rank] < global_num_end
           || this_slice->use_next_global_num == false) {
 
-        if (this_slice->safe_mode == true) {
-          MPI_Send(&distant_rank, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm);
-          MPI_Recv(&n_distant_entities, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm, &status);
-        }
-        else {
-          MPI_Probe(distant_rank, FVM_MPI_TAG, comm, &status);
-          MPI_Get_count(&status, FVM_MPI_GNUM, &n_distant_entities);
-        }
+        MPI_Send(&distant_rank, 1, MPI_INT, distant_rank, FVM_MPI_TAG, comm);
+        MPI_Recv(&n_distant_entities, 1, MPI_INT,
+                 distant_rank, FVM_MPI_TAG, comm, &status);
 
         MPI_Recv(displacements, n_distant_entities, FVM_MPI_GNUM,
                  distant_rank, FVM_MPI_TAG, comm, &status);
@@ -1430,14 +1389,12 @@ fvm_gather_strided_connect(const fvm_lnum_t    local_connect[],
 
       /* Send local index to rank zero */
 
-      if (this_slice->safe_mode == true) {
-        int buf_val;
-        MPI_Recv(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm, &status);
-        buf_val = n_local_entities + 1;
-        MPI_Send(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm);
-      }
+      int buf_val;
+
+      MPI_Recv(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm, &status);
+
+      buf_val = n_local_entities + 1;
+      MPI_Send(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm);
 
       MPI_Send(displacements, n_local_entities + 1, FVM_MPI_GNUM, 0,
                FVM_MPI_TAG, comm);
@@ -1640,16 +1597,9 @@ fvm_gather_indexed_numbers(const fvm_lnum_t     local_index[],
 
         size_t  recv_size = 0;
 
-        if (this_slice->safe_mode == true) {
-          MPI_Send(&distant_rank, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm);
-          MPI_Recv(&n_distant_entities, 1, MPI_INT,
-                   distant_rank, FVM_MPI_TAG, comm, &status);
-        }
-        else {
-          MPI_Probe(distant_rank, FVM_MPI_TAG, comm, &status);
-          MPI_Get_count(&status, FVM_MPI_GNUM, &n_distant_entities);
-        }
+        MPI_Send(&distant_rank, 1, MPI_INT, distant_rank, FVM_MPI_TAG, comm);
+        MPI_Recv(&n_distant_entities, 1, MPI_INT,
+                 distant_rank, FVM_MPI_TAG, comm, &status);
 
       /* Get index from distant rank */
 
@@ -1701,14 +1651,13 @@ fvm_gather_indexed_numbers(const fvm_lnum_t     local_index[],
 
       /* Send local index to rank zero */
 
-      if (this_slice->safe_mode == true) {
-        int buf_val;
-        MPI_Recv(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm, &status);
-        buf_val = n_local_entities + 1;
-        MPI_Send(&buf_val, 1, MPI_INT,
-                 0, FVM_MPI_TAG, comm);
-      }
+      int buf_val;
+
+      MPI_Recv(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm, &status);
+
+      buf_val = n_local_entities + 1;
+      MPI_Send(&buf_val, 1, MPI_INT, 0, FVM_MPI_TAG, comm);
+
       MPI_Send(displacements, n_local_entities + 1, FVM_MPI_GNUM,
                0, FVM_MPI_TAG, comm);
 
