@@ -19,7 +19,7 @@
   This file is part of the "Finite Volume Mesh" library, intended to provide
   finite volume mesh and associated fields I/O and manipulation services.
 
-  Copyright (C) 2004-2007  EDF
+  Copyright (C) 2004-2009  EDF
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -57,6 +57,7 @@
 
 #include "fvm_defs.h"
 #include "fvm_config_defs.h"
+#include "fvm_morton.h"
 #include "fvm_order.h"
 #include "fvm_parall.h"
 
@@ -128,6 +129,126 @@ struct _fvm_io_num_t {
  * Private function definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Use bubble sort on an expectedly short sequence of coordinates
+ * to ensure lexicographical ordering.
+ *
+ * parameters:
+ *   dim        <-- spatial dimension
+ *   start_id   <-- start id in array
+ *   end_id     <-- past-the-end id in array
+ *   coords     <-- pointer to entity coordinates (interlaced)
+ *   order      <-> ordering array base on Morton encoding, or
+ *                  lexicographical coordinate ordering for ties
+ *----------------------------------------------------------------------------*/
+
+inline static void
+_reorder_coords_lexicographic(int                dim,
+                              size_t             start_id,
+                              size_t             end_id,
+                              const fvm_coord_t  coords[],
+                              fvm_lnum_t         order[])
+{
+  size_t  i;
+  _Bool g_swap;
+
+  do {
+
+    g_swap = false;
+
+    for (i = start_id + 1; i < end_id; i++) {
+
+      size_t j_prev = order[i-1], j = order[i];
+      _Bool l_swap = false;
+
+      if (dim == 3) {
+        if (coords[j_prev*3] < coords[j*3])
+          continue;
+        else if (coords[j_prev*3] > coords[j*3])
+          l_swap = true;
+        else if (coords[j_prev*3 + 1] < coords[j*3 + 1])
+          continue;
+        else if (   coords[j_prev*3 + 1] > coords[j*3 + 1]
+                 || coords[j_prev*3 + 2] > coords[j*3 + 2])
+          l_swap = true;
+      }
+      else if (dim == 2) {
+        if (coords[j_prev*2] < coords[j*2 + 1])
+          continue;
+        else if (   coords[j_prev*2]     > coords[j*2]
+                 || coords[j_prev*2 + 1] > coords[j*2 + 1])
+          l_swap = true;
+      }
+      else { /* if (dim == 1) */
+        if (coords[j_prev] > coords[j])
+          l_swap = true;
+      }
+
+      if (l_swap) {
+        fvm_lnum_t o_save = order[i-1];
+        order[i-1] = order[i];
+        order[i] = o_save;
+        g_swap = true;
+      }
+    }
+
+  } while (g_swap);
+}
+
+/*----------------------------------------------------------------------------
+ * Creation of an I/O numbering structure based on coordinates.
+ *
+ * The ordering is based on a Morton code, and it is expected that
+ * entities are unique (i.e. not duplicated on 2 or more ranks).
+ * In the case that 2 entities have a same Morton code, their global
+ * number will be different, but their order is undetermined.
+ *
+ * parameters:
+ *   dim        <-- spatial dimension
+ *   n_entities <-- number of entities considered
+ *   coords     <-- pointer to entity coordinates (interlaced)
+ *   m_code     <-- Morton code associated with each entity
+ *   order      <-> ordering array base on Morton encoding, or
+ *                  lexicographical coordinate ordering for ties
+ *
+ * returns:
+ *  pointer to I/O numbering structure
+ *----------------------------------------------------------------------------*/
+
+static void
+_check_morton_ordering(int                      dim,
+                       size_t                   n_entities,
+                       const fvm_coord_t        coords[],
+                       const fvm_morton_code_t  m_code[],
+                       fvm_lnum_t               order[])
+{
+  size_t  i_prev = 0, i = 1;
+
+  /* Check ordering; if two entities have the same Morton codes,
+     use lexicographical coordinates ordering to ensure the
+     final order is deterministic. */
+
+  for (i = 1; i < n_entities; i++) {
+
+    size_t j_prev = order[i_prev], j = order[i];
+
+    if (   m_code[j_prev].X[0] != m_code[j].X[0]
+        || m_code[j_prev].X[1] != m_code[j].X[1]
+        || m_code[j_prev].X[2] != m_code[j].X[2]) {
+
+      /* If successive values have the same Morton code,
+         order them lexicographically */
+      if (i_prev < i - 1)
+        _reorder_coords_lexicographic(dim, i_prev, i-1, coords, order);
+
+    }
+    i_prev = i;
+  }
+
+  if (i_prev < n_entities - 1)
+    _reorder_coords_lexicographic(dim, i_prev, n_entities - 1, coords, order);
+}
+
 #if defined(FVM_HAVE_MPI)
 
 /*----------------------------------------------------------------------------
@@ -480,14 +601,10 @@ _fvm_io_num_global_order(fvm_io_num_t       *this_io_num,
  * be sorted, but need not be contiguous. On output, the numbering
  * will be contiguous.
  *
- * As an option, a number of sub-entities per initial entity may be
- * given, in which case sub-entities of a same entity will have contiguous
- * numbers in the final ordering.
- *
  * parameters:
  *   this_io_num    <-> pointer to structure that should be ordered
- *   n_sub_entities <-- optional number of sub-entities per initial entity,
- *                      or NULL if unused
+ *   stride         <-- values per entity
+ *   global_num     <-- global numbering array
  *   comm           <-- associated MPI communicator
  *----------------------------------------------------------------------------*/
 
@@ -502,7 +619,7 @@ _fvm_io_num_global_order_s(fvm_io_num_t       *this_io_num,
   int         rank;
 
   fvm_gnum_t  *block_global_num = NULL, *recv_global_num = NULL;
-  fvm_lnum_t  *recv_n_sub = NULL, *recv_order = NULL;
+  fvm_lnum_t  *recv_order = NULL;
   int         *send_count = NULL, *recv_count = NULL;
   int         *send_shift = NULL, *recv_shift = NULL;
 
@@ -523,7 +640,7 @@ _fvm_io_num_global_order_s(fvm_io_num_t       *this_io_num,
     size_t      n_ent = this_io_num->global_num_size;
 
     if (n_ent > 0)
-      local_max = global_num[(n_ent-1)*2];
+      local_max = global_num[(n_ent-1)*stride];
     MPI_Allreduce(&local_max, &global_max, 1, FVM_MPI_GNUM, MPI_MAX, comm);
     this_io_num->global_count = global_max;
   }
@@ -613,7 +730,6 @@ _fvm_io_num_global_order_s(fvm_io_num_t       *this_io_num,
 
   /* Partial clean-up */
 
-  BFT_FREE(recv_n_sub);
   BFT_FREE(recv_order);
   BFT_FREE(recv_global_num);
 
@@ -659,6 +775,300 @@ _fvm_io_num_global_order_s(fvm_io_num_t       *this_io_num,
   this_io_num->global_count = _fvm_io_num_global_max(this_io_num, comm);
 }
 
+/*----------------------------------------------------------------------------
+ * Compare two elements in an indexed list and returns true if element in
+ * position i1 is strictly greater than element in position i2.
+ *
+ * parameters:
+ *   i1        <-- position in index for the first element
+ *   i2        <-- position in index for the second element
+ *   index     <-- number of values to compare for each entity
+ *   number    <-- pointer to numbers of entities that should be ordered.
+ *                 (if NULL, a default 1 to n numbering is considered)
+ *
+ * returns:
+ *  true or false
+ *----------------------------------------------------------------------------*/
+
+inline static _Bool
+_indexed_is_greater(size_t            i1,
+                    size_t            i2,
+                    const fvm_lnum_t  index[],
+                    const fvm_gnum_t  number[])
+{
+  int  i;
+
+  fvm_lnum_t  i1_s = index[i1], i1_e = index[i1+1], s1 = i1_e - i1_s;
+  fvm_lnum_t  i2_s = index[i2], i2_e = index[i2+1], s2 = i2_e - i2_s;
+
+  if (s1 > s2) {
+
+    for (i = 0; i < s2; i++) {
+      if (number[i1_s + i] > number[i2_s + i])
+        return true;
+      else if (number[i1_s + i] < number[i2_s + i])
+        return false;
+    }
+
+    return true;
+  }
+  else { /* s1 <= s2 */
+
+    for (i = 0; i < s1; i++) {
+      if (number[i1_s + i] > number[i2_s + i])
+        return true;
+      else if (number[i1_s + i] < number[i2_s + i])
+        return false;
+    }
+
+    return false;
+  }
+
+}
+
+/*----------------------------------------------------------------------------
+ * Global indexed ordering associated with an I/O numbering structure.
+ *
+ * The structure should contain an initial ordering, which should
+ * be sorted, but need not be contiguous. On output, the numbering
+ * will be contiguous.
+ *
+ * parameters:
+ *   this_io_num    <-> pointer to structure that should be ordered
+ *   index          <-- index on entities for global_num[]
+ *   global_num     <--
+ *   comm           <-- associated MPI communicator
+ *----------------------------------------------------------------------------*/
+
+static void
+_fvm_io_num_global_order_index(fvm_io_num_t       *this_io_num,
+                               fvm_lnum_t          index[],
+                               fvm_gnum_t          global_num[],
+                               MPI_Comm            comm)
+{
+  int  rank, local_rank, size;
+  size_t  i, shift, slice_size;
+
+  fvm_gnum_t  n_ent_recv = 0, current_global_num = 0, global_num_shift = 0;
+  int  *send_count = NULL, *recv_count = NULL;
+  int  *send_shift = NULL, *recv_shift = NULL;
+  fvm_lnum_t  *recv_order = NULL,  *recv_index = NULL, *send_buffer = NULL;
+  fvm_gnum_t  *block_global_num = NULL, *recv_global_num = NULL;
+
+  /* Initialization */
+
+  MPI_Comm_rank(comm, &local_rank);
+  MPI_Comm_size(comm, &size);
+
+  /* Get maximum global number value for first value of each series
+     (does not need to be exact, simply used to define blocks) */
+
+  {
+    fvm_gnum_t  local_max = 0, global_max = 0;
+    size_t      n_ent = this_io_num->global_num_size;
+
+    if (n_ent > 0)
+      local_max = global_num[index[n_ent-1]];
+    MPI_Allreduce(&local_max, &global_max, 1, FVM_MPI_GNUM, MPI_MAX, comm);
+    this_io_num->global_count = global_max;
+  }
+
+  /* slice_size = ceil(this_io_num->global_count/size) */
+
+  slice_size = this_io_num->global_count / size;
+  if (this_io_num->global_count % size > 0)
+    slice_size += 1;
+
+  /* Build for each slice, a new ordered indexed list from the received
+     elements */
+
+  assert(sizeof(fvm_gnum_t) >= sizeof(fvm_lnum_t));
+
+  BFT_MALLOC(send_count, size, int);
+  BFT_MALLOC(recv_count, size, int);
+  BFT_MALLOC(send_shift, size + 1, int);
+  BFT_MALLOC(recv_shift, size + 1, int);
+
+  /* Count number of values to send to each process */
+
+  for (rank = 0; rank < size; rank++)
+    send_count[rank] = 0;
+
+  for (i = 0; i < (size_t)(this_io_num->global_num_size); i++) {
+    rank = (global_num[index[i]] - 1) / slice_size;
+    send_count[rank] += 1;
+  }
+
+  MPI_Alltoall(send_count, 1, MPI_INT, recv_count, 1, MPI_INT, comm);
+
+  send_shift[0] = 0;
+  recv_shift[0] = 0;
+
+  for (rank = 0; rank < size; rank++) {
+    send_shift[rank+1] = send_shift[rank] + send_count[rank];
+    recv_shift[rank+1] = recv_shift[rank] + recv_count[rank];
+  }
+
+  /* Get recv_index */
+
+  n_ent_recv = recv_shift[size];
+  BFT_MALLOC(recv_index, n_ent_recv + 1, fvm_lnum_t);
+  BFT_MALLOC(send_buffer, send_shift[size], fvm_lnum_t);
+
+  assert(send_shift[size] == this_io_num->global_num_size);
+
+  for (rank = 0; rank < size; rank++)
+    send_count[rank] = 0;
+
+  for (i = 0; i < (size_t)this_io_num->global_num_size; i++) {
+
+    rank = (global_num[index[i]] - 1) / slice_size;
+    shift = send_shift[rank] + send_count[rank];
+    send_buffer[shift] = index[i+1] - index[i];
+    send_count[rank] +=  1;
+
+  }
+
+  MPI_Alltoallv(send_buffer, send_count, send_shift, FVM_MPI_LNUM,
+                &(recv_index[1]), recv_count, recv_shift, FVM_MPI_LNUM, comm);
+
+  recv_index[0] = 0;
+  for (i = 0; i < n_ent_recv; i++)
+    recv_index[i+1] += recv_index[i];
+
+  BFT_FREE(send_buffer);
+
+  /* Get recv_global_num */
+
+  /* Count number of values to send to each process */
+
+  for (rank = 0; rank < size; rank++)
+    send_count[rank] = 0;
+
+  for (i = 0; i < (size_t)(this_io_num->global_num_size); i++) {
+    rank = (global_num[index[i]] - 1) / slice_size;
+    send_count[rank] += index[i+1] - index[i];
+  }
+
+  MPI_Alltoall(send_count, 1, MPI_INT, recv_count, 1, MPI_INT, comm);
+
+  send_shift[0] = 0;
+  recv_shift[0] = 0;
+
+  for (rank = 0; rank < size; rank++) {
+    send_shift[rank+1] = send_shift[rank] + send_count[rank];
+    recv_shift[rank+1] = recv_shift[rank] + recv_count[rank];
+  }
+
+  assert(send_shift[size] == index[this_io_num->global_num_size]);
+  assert(recv_shift[size] == recv_index[n_ent_recv]);
+
+  BFT_MALLOC(recv_global_num, recv_index[n_ent_recv], fvm_gnum_t);
+
+  /* As data is sorted by increasing base global numbering, we do not
+     need to build an extra array, but only to send the correct parts
+     of the indexed list to the correct processors */
+
+  MPI_Alltoallv(global_num, send_count, send_shift, FVM_MPI_GNUM,
+                recv_global_num, recv_count, recv_shift, FVM_MPI_GNUM, comm);
+
+  if (n_ent_recv > 0) { /* Order received elements of the indexed list */
+
+    size_t prev_id, cur_id;
+
+    BFT_MALLOC(recv_order, n_ent_recv, fvm_lnum_t);
+
+    fvm_order_local_allocated_i(NULL,
+                                recv_global_num,
+                                recv_index,
+                                recv_order,
+                                n_ent_recv);
+
+    /* Determine global order; requires ordering to loop through buffer by
+       increasing number (slice blocks associated with each process are
+       already sorted, but the whole "gathered" slice is not).
+       We build an initial global order based on the initial global numbering,
+       such that for each slice, the global number of an entity is equal to
+       the cumulative number of elements */
+
+    BFT_MALLOC(block_global_num, n_ent_recv, fvm_gnum_t);
+
+    current_global_num = 1;
+    prev_id = recv_order[0];
+    block_global_num[recv_order[0]] = current_global_num;
+
+    for (i = 1; i < n_ent_recv; i++) {
+
+      cur_id = recv_order[i];
+
+      if (_indexed_is_greater(cur_id, prev_id, recv_index, recv_global_num))
+        current_global_num += 1;
+
+      block_global_num[recv_order[i]] = current_global_num;
+      prev_id = cur_id;
+
+    }
+
+  } /* End if n_ent_recv > 0 */
+
+  /* Partial clean-up */
+
+  BFT_FREE(recv_order);
+  BFT_FREE(recv_index);
+  BFT_FREE(recv_global_num);
+
+  /* At this stage, block_global_num[] is valid for this process, and
+     current_global_num indicates the total number of entities handled
+     by this process; we must now shift global numberings on different
+     processes by the cumulative total number of entities handled by
+     each process */
+
+  MPI_Scan(&current_global_num, &global_num_shift, 1, FVM_MPI_GNUM,
+           MPI_SUM, comm);
+  global_num_shift -= current_global_num;
+
+  for (i = 0; i < n_ent_recv; i++)
+    block_global_num[i] += global_num_shift;
+
+  /* Return global order to all processors */
+
+  /* Count number of values to send to each process */
+
+  for (rank = 0; rank < size; rank++)
+    send_count[rank] = 0;
+
+  for (i = 0; i < (size_t)(this_io_num->global_num_size); i++) {
+    rank = (global_num[index[i]] - 1) / slice_size;
+    send_count[rank] += 1;
+  }
+
+  MPI_Alltoall(send_count, 1, MPI_INT, recv_count, 1, MPI_INT, comm);
+
+  send_shift[0] = 0;
+  recv_shift[0] = 0;
+
+  for (rank = 0; rank < size; rank++) {
+    send_shift[rank+1] = send_shift[rank] + send_count[rank];
+    recv_shift[rank+1] = recv_shift[rank] + recv_count[rank];
+  }
+
+  MPI_Alltoallv(block_global_num, recv_count, recv_shift, FVM_MPI_GNUM,
+                this_io_num->_global_num, send_count, send_shift, FVM_MPI_GNUM,
+                comm);
+
+  /* Free memory */
+
+  BFT_FREE(block_global_num);
+  BFT_FREE(send_count);
+  BFT_FREE(recv_count);
+  BFT_FREE(send_shift);
+  BFT_FREE(recv_shift);
+
+  /* Get final maximum global number value */
+
+  this_io_num->global_count = _fvm_io_num_global_max(this_io_num, comm);
+}
+
 /*----------------------------------------------------------------------------
  * Return the global number of sub-entities associated with an initial
  * entity whose global numbering is known, given the number of
@@ -994,34 +1404,34 @@ fvm_io_num_create_from_sub(const fvm_io_num_t  *base_io_num,
   assert(fvm_parall_get_size() > 1); /* Otherwise, base_io_num should be NULL */
 
 #if defined(FVM_HAVE_MPI)
- {
-   fvm_lnum_t  i, n_ent;
+  {
+    fvm_lnum_t  i, n_ent;
 
-   /* Create structure */
+    /* Create structure */
 
-   BFT_MALLOC(this_io_num, 1, fvm_io_num_t);
+    BFT_MALLOC(this_io_num, 1, fvm_io_num_t);
 
-   n_ent = base_io_num->global_num_size;
+    n_ent = base_io_num->global_num_size;
 
-   BFT_MALLOC(this_io_num->_global_num, n_ent, fvm_gnum_t);
-   this_io_num->global_num = this_io_num->_global_num;
+    BFT_MALLOC(this_io_num->_global_num, n_ent, fvm_gnum_t);
+    this_io_num->global_num = this_io_num->_global_num;
 
-   this_io_num->global_num_size = n_ent;
+    this_io_num->global_num_size = n_ent;
 
-   /* Assign initial global numbers */
+    /* Assign initial global numbers */
 
-   for (i = 0 ; i < n_ent ; i++)
-     this_io_num->_global_num[i] = base_io_num->global_num[i];
+    for (i = 0 ; i < n_ent ; i++)
+      this_io_num->_global_num[i] = base_io_num->global_num[i];
 
-   /* Order globally */
+    /* Order globally */
 
-   this_io_num->global_count = n_ent;
+    this_io_num->global_count = n_ent;
 
-   _fvm_io_num_copy_on_write(this_io_num);
-   _fvm_io_num_global_order(this_io_num,
-                            n_sub_entities,
-                            fvm_parall_get_mpi_comm());
- }
+    _fvm_io_num_copy_on_write(this_io_num);
+    _fvm_io_num_global_order(this_io_num,
+                             n_sub_entities,
+                             fvm_parall_get_mpi_comm());
+  }
 #endif
 
   return this_io_num;
@@ -1050,8 +1460,6 @@ fvm_io_num_create_from_adj_s(const fvm_lnum_t  parent_entity_number[],
                              size_t            n_entities,
                              size_t            stride)
 {
-  fvm_gnum_t *_adjacency = NULL;
-
   fvm_io_num_t  *this_io_num = NULL;
 
   /* Initial checks */
@@ -1065,6 +1473,221 @@ fvm_io_num_create_from_adj_s(const fvm_lnum_t  parent_entity_number[],
                                 n_entities) == true);
 
 #if defined(FVM_HAVE_MPI)
+  {
+    fvm_gnum_t *_adjacency = NULL;
+
+    /* Create structure */
+
+    BFT_MALLOC(this_io_num, 1, fvm_io_num_t);
+
+    this_io_num->global_num_size = n_entities;
+
+    BFT_MALLOC(this_io_num->_global_num, n_entities, fvm_gnum_t);
+    this_io_num->global_num = this_io_num->_global_num;
+
+    if (n_entities > 0) {
+
+      size_t  i, j;
+
+      /* Assign initial global numbers */
+
+      BFT_MALLOC(_adjacency, n_entities*stride, fvm_gnum_t);
+
+      if (parent_entity_number != NULL) {
+        for (i = 0 ; i < n_entities ; i++) {
+          for (j = 0; j < stride; j++)
+            _adjacency[i*stride + j]
+              = adjacency[(parent_entity_number[i]-1)*stride + j];
+        }
+      }
+      else
+        memcpy(_adjacency, adjacency, n_entities*stride*sizeof(fvm_gnum_t));
+
+    }
+
+    /* Order globally */
+
+    this_io_num->global_count = n_entities;
+
+    _fvm_io_num_global_order_s(this_io_num,
+                               stride,
+                               _adjacency,
+                               fvm_parall_get_mpi_comm());
+
+    BFT_FREE(_adjacency);
+  }
+#endif
+
+  return this_io_num;
+}
+
+/*----------------------------------------------------------------------------
+ * Creation of an I/O numbering structure based on an indexed adjacency.
+ *
+ * The corresponding entities do not need to be locally ordered.
+ *
+ * parameters:
+ *  parent_entity_number <-- pointer to list of selected entitie's parent's
+ *                           numbers, or NULL if all first n_ent entities
+ *                           are used
+ *  index                <-- index on entities for adjacency
+ *  adjacency            <-- entity adjacency (1 to n global numbering)
+ *  n_entities           <-- number of entities considered
+ *
+ * returns:
+ *  pointer to I/O numbering structure
+ *----------------------------------------------------------------------------*/
+
+fvm_io_num_t *
+fvm_io_num_create_from_adj_i(const fvm_lnum_t  parent_entity_number[],
+                             const fvm_lnum_t  index[],
+                             const fvm_gnum_t  adjacency[],
+                             fvm_lnum_t        n_entities)
+{
+  fvm_io_num_t  *this_io_num = NULL;
+
+  /* Initial checks */
+
+  if (fvm_parall_get_size() < 2)
+    return NULL;
+
+#if defined(FVM_HAVE_MPI)
+  {
+    fvm_lnum_t  *_index = NULL;
+    fvm_gnum_t *_adjacency = NULL;
+
+#if defined(DEBUG) && !defined(NDEBUG)
+    const char no_adjacent_elt_msg[]
+      = " Error during the creation of a fvm_io_num_t structure\n"
+        " for an indexed adjacency.\n\n"
+        " At least one entity has no adjacent element, and\n"
+        " this case is not currently handled.\n\n"
+        " Check if this definition is correct.";
+#endif
+
+    /* Create structure */
+
+    BFT_MALLOC(this_io_num, 1, fvm_io_num_t);
+
+    this_io_num->global_num_size = n_entities;
+
+    BFT_MALLOC(this_io_num->_global_num, n_entities, fvm_gnum_t);
+    this_io_num->global_num = this_io_num->_global_num;
+
+    if (n_entities > 0) {
+
+      fvm_lnum_t  i, j, k, ent_id, _shift;
+
+      /* Assign initial global numbers */
+
+      BFT_MALLOC(_index, n_entities + 1, fvm_lnum_t);
+
+      if (parent_entity_number != NULL) {
+
+#if defined(DEBUG) && !defined(NDEBUG)
+        for (i = 0 ; i < n_entities ; i++) {
+          ent_id = parent_entity_number[i]-1;
+          if ((index[ent_id+1] - index[ent_id]) == 0)
+            bft_error(__FILE__, __LINE__, 0, no_adjacent_elt_msg);
+        }
+#endif
+
+        /* Count reduced size */
+
+        for (i = 0 ; i < n_entities ; i++) {
+          ent_id = parent_entity_number[i]-1;
+          _index[i+1] = index[ent_id+1] - index[ent_id];
+        }
+
+        _index[0] = 0;
+        for (i = 0 ; i < n_entities ; i++)
+          _index[i+1] += _index[i];
+
+        BFT_MALLOC(_adjacency, _index[n_entities], fvm_gnum_t);
+
+        /* Define reduced index and adjacency */
+
+        for (i = 0 ; i < n_entities ; i++) {
+
+          ent_id = parent_entity_number[i]-1;
+          _shift = _index[i];
+
+          for (j = index[ent_id], k = 0; j < index[ent_id+1]; j++, k++)
+            _adjacency[_shift + k] = adjacency[j];
+
+        }
+
+      }
+      else {
+
+#if defined(DEBUG) && !defined(NDEBUG)
+        for (i = 0 ; i < n_entities ; i++) {
+          if ((index[i+1] - index[i]) == 0)
+            bft_error(__FILE__, __LINE__, 0, no_adjacent_elt_msg);
+        }
+#endif
+
+        BFT_MALLOC(_adjacency, index[n_entities], fvm_gnum_t);
+
+        memcpy(_index, index, (n_entities+1)*sizeof(fvm_lnum_t));
+        memcpy(_adjacency, adjacency, index[n_entities]*sizeof(fvm_gnum_t));
+
+      }
+
+    }
+
+    /* Order globally */
+
+    this_io_num->global_count = n_entities;
+
+    _fvm_io_num_global_order_index(this_io_num,
+                                   _index,
+                                   _adjacency,
+                                   fvm_parall_get_mpi_comm());
+
+    BFT_FREE(_adjacency);
+    BFT_FREE(_index);
+  }
+#endif
+
+  return this_io_num;
+}
+
+/*----------------------------------------------------------------------------
+ * Creation of an I/O numbering structure based on coordinates.
+ *
+ * The ordering is based on a Morton code, and it is expected that
+ * entities are unique (i.e. not duplicated on 2 or more ranks).
+ * In the case that 2 entities have a same Morton code, their global
+ * number will be determined by lexicographical ordering of coordinates.
+ *
+ * parameters:
+ *   coords     <-- pointer to entity coordinates (interlaced)
+ *   dim        <-- spatial dimension
+ *   n_entities <-- number of entities considered
+ *
+ * returns:
+ *  pointer to I/O numbering structure
+ *----------------------------------------------------------------------------*/
+
+fvm_io_num_t *
+fvm_io_num_create_from_coords(const fvm_coord_t  coords[],
+                              int                dim,
+                              size_t             n_entities)
+{
+  size_t  i;
+  fvm_coord_t extents[6];
+  fvm_lnum_t  *order = NULL;
+  fvm_morton_code_t *m_code = NULL;
+
+#if defined(FVM_HAVE_MPI)
+  MPI_Comm comm = fvm_parall_get_mpi_comm();
+#endif
+
+  const int level = sizeof(fvm_morton_int_t)*8 - 1;
+  const int n_ranks = fvm_parall_get_size();
+
+  fvm_io_num_t  *this_io_num = NULL;
 
   /* Create structure */
 
@@ -1075,38 +1698,216 @@ fvm_io_num_create_from_adj_s(const fvm_lnum_t  parent_entity_number[],
   BFT_MALLOC(this_io_num->_global_num, n_entities, fvm_gnum_t);
   this_io_num->global_num = this_io_num->_global_num;
 
-  if (n_entities > 0) {
+  /* Build Morton encoding and order it */
 
-    size_t  i, j;
+#if defined(FVM_HAVE_MPI)
+  fvm_morton_get_coord_extents(dim, n_entities, coords, extents, comm);
+#else
+  fvm_morton_get_coord_extents(dim, n_entities, coords, extents);
+#endif
 
-    /* Assign initial global numbers */
+  BFT_MALLOC(m_code, n_entities, fvm_morton_code_t);
+  BFT_MALLOC(order, n_entities, fvm_lnum_t);
 
-    BFT_MALLOC(_adjacency, n_entities*stride, fvm_gnum_t);
+  fvm_morton_encode_coords(dim, level, extents, n_entities, coords, m_code);
 
-    if (parent_entity_number != NULL) {
-      for (i = 0 ; i < n_entities ; i++) {
-        for (j = 0; j < stride; j++)
-          _adjacency[i*stride + j]
-            = adjacency[(parent_entity_number[i]-1)*stride + j];
-      }
+  fvm_morton_local_order(dim, n_entities, m_code, order);
+
+#if defined(FVM_HAVE_MPI)
+
+  if (n_ranks > 1) {
+
+    int rank_id;
+    fvm_lnum_t j, shift;
+
+    size_t n_block_ents = 0;
+    fvm_gnum_t current_global_num = 0, global_num_shift = 0;
+    double fit = 0.;
+
+    int *c_rank = NULL;
+    int *send_count = NULL, *send_shift = NULL;
+    int *recv_count = NULL, *recv_shift = NULL;
+    fvm_coord_t *send_coords = NULL, *recv_coords = NULL;
+    fvm_lnum_t *weight = NULL;
+    fvm_gnum_t *block_global_num = NULL;
+    fvm_morton_code_t *morton_index = NULL;
+
+    BFT_MALLOC(weight, n_entities, fvm_lnum_t);
+    BFT_MALLOC(morton_index, n_ranks + 1, fvm_morton_code_t);
+
+    for (i = 0; i < n_entities; i++)
+      weight[i] = 1;
+
+    fit = fvm_morton_build_rank_index(dim,
+                                      level,
+                                      n_entities,
+                                      m_code,
+                                      weight,
+                                      order,
+                                      morton_index,
+                                      comm);
+
+    BFT_FREE(order);
+    BFT_FREE(weight);
+    BFT_MALLOC(c_rank, n_entities, int);
+
+    for (i = 0; i < n_entities; i++)
+      c_rank[i] = fvm_morton_quantile_search(dim,
+                                             n_ranks,
+                                             m_code[i],
+                                             morton_index);
+
+    BFT_FREE(morton_index);
+    BFT_FREE(m_code);
+
+    /* Build send_buf, send_count and send_shift
+       to build a rank to coords indexed list */
+
+    BFT_MALLOC(send_count, n_ranks, int);
+    BFT_MALLOC(recv_count, n_ranks, int);
+    BFT_MALLOC(send_shift, n_ranks + 1, int);
+    BFT_MALLOC(recv_shift, n_ranks + 1, int);
+
+    for (rank_id = 0; rank_id < n_ranks; rank_id++)
+      send_count[rank_id] = 0;
+
+    for (i = 0; i < n_entities; i++)
+      send_count[c_rank[i]] += dim;
+
+    /* Exchange number of coords to send to each process */
+
+    MPI_Alltoall(send_count, 1, MPI_INT, recv_count, 1, MPI_INT, comm);
+
+    send_shift[0] = 0;
+    recv_shift[0] = 0;
+    for (rank_id = 0; rank_id < n_ranks; rank_id++) {
+      send_shift[rank_id + 1] = send_shift[rank_id] + send_count[rank_id];
+      recv_shift[rank_id + 1] = recv_shift[rank_id] + recv_count[rank_id];
     }
-    else
-      memcpy(_adjacency, adjacency, n_entities*stride*sizeof(fvm_gnum_t));
+
+    /* Build send and receive buffers */
+
+    BFT_MALLOC(send_coords, send_shift[n_ranks], fvm_coord_t);
+
+    for (rank_id = 0; rank_id < n_ranks; rank_id++)
+      send_count[rank_id] = 0;
+
+    for (i = 0; i < n_entities; i++) {
+      rank_id = c_rank[i];
+      shift = send_shift[rank_id] + send_count[rank_id];
+      for (j = 0; j < dim; j++)
+        send_coords[shift + j] = coords[i*dim + j];
+      send_count[rank_id] += dim;
+    }
+
+    BFT_FREE(c_rank);
+    BFT_MALLOC(recv_coords, recv_shift[n_ranks], fvm_coord_t);
+
+    /* Exchange coords between processes */
+
+    MPI_Alltoallv(send_coords, send_count, send_shift, FVM_MPI_COORD,
+                  recv_coords, recv_count, recv_shift, FVM_MPI_COORD,
+                  comm);
+
+    BFT_FREE(send_coords);
+
+    /* Now re-build Morton codes on block distribution */
+
+    n_block_ents = recv_shift[n_ranks] / dim;
+
+    BFT_MALLOC(m_code, n_block_ents, fvm_morton_code_t);
+    BFT_MALLOC(order, n_block_ents, fvm_lnum_t);
+
+    fvm_morton_encode_coords(dim,
+                             level,
+                             extents,
+                             n_block_ents,
+                             recv_coords,
+                             m_code);
+
+    fvm_morton_local_order(dim, n_block_ents, m_code, order);
+
+    /* Check ordering; if two entities have the same Morton codes,
+       use lexicographical coordinates ordering to ensure the
+       final order is deterministic. */
+
+    _check_morton_ordering(dim, n_block_ents, recv_coords, m_code, order);
+
+    /* Determine global order; requires ordering to loop through buffer by
+       increasing number (slice blocks associated with each process are
+       already sorted, but the whole "gathered" slice is not).
+       We build an initial global order based on the initial global numbering,
+       such that for each slice, the global number of an entity is equal to
+       the cumulative number of sub-entities */
+
+    BFT_FREE(m_code);
+    BFT_FREE(recv_coords);
+    BFT_MALLOC(block_global_num, n_block_ents, fvm_gnum_t);
+
+    for (i = 0; i < n_block_ents; i++)
+      block_global_num[order[i]] = i+1;
+
+    BFT_FREE(order);
+
+    current_global_num = n_block_ents;
+
+    /* At this stage, block_global_num[] is valid for this process, and
+       current_global_num indicates the total number of entities handled
+       by this process; we must now shift global numberings on different
+       processes by the cumulative total number of entities handled by
+       each process */
+
+    MPI_Scan(&current_global_num, &global_num_shift, 1, FVM_MPI_GNUM,
+             MPI_SUM, comm);
+    global_num_shift -= current_global_num;
+
+    for (i = 0; i < n_block_ents; i++)
+      block_global_num[i] += global_num_shift;
+
+    /* Return global order to all processors */
+
+    for (rank_id = 0; rank_id < n_ranks; rank_id++) {
+      send_count[rank_id] /= dim;
+      recv_count[rank_id] /= dim;
+      send_shift[rank_id] /= dim;
+      recv_shift[rank_id] /= dim;
+    }
+
+    MPI_Alltoallv(block_global_num, recv_count, recv_shift, FVM_MPI_GNUM,
+                  this_io_num->_global_num, send_count, send_shift, FVM_MPI_GNUM,
+                  comm);
+
+    /* Free memory */
+
+    BFT_FREE(block_global_num);
+
+    BFT_FREE(send_count);
+    BFT_FREE(recv_count);
+    BFT_FREE(send_shift);
+    BFT_FREE(recv_shift);
+
+    /* Get final maximum global number value */
+
+    this_io_num->global_count = _fvm_io_num_global_max(this_io_num, comm);
 
   }
 
-  /* Order globally */
+#endif /* FVM_HAVE_MPI */
 
-  this_io_num->global_count = n_entities;
+  if (n_ranks == 1) {
 
-  _fvm_io_num_global_order_s(this_io_num,
-                             stride,
-                             _adjacency,
-                             fvm_parall_get_mpi_comm());
+    _check_morton_ordering(dim, n_entities, coords, m_code, order);
 
-  BFT_FREE(_adjacency);
+    BFT_FREE(m_code);
 
-#endif
+    for (i = 1; i < n_entities; i++)
+      this_io_num->_global_num[order[i]] = i+1;
+
+    BFT_FREE(order);
+
+    this_io_num->global_count = n_entities;
+
+  }
 
   return this_io_num;
 }
@@ -1251,26 +2052,26 @@ fvm_io_num_dump(const fvm_io_num_t  *const this_io_num)
   fvm_lnum_t i;
 
   if (this_io_num == NULL) {
-    bft_printf(_("  global numbering: nil\n"));
+    bft_printf("  global numbering: nil\n");
     return;
   }
 
-  bft_printf(_("  global numbering size:            %u\n"),
+  bft_printf("  global numbering size:            %u\n",
              (unsigned)this_io_num->global_num_size);
 
-  bft_printf(_("\n"
-               "  pointer to shareable array:\n"
-               "    global_num:                     %p\n"),
+  bft_printf("\n"
+             "  pointer to shareable array:\n"
+             "    global_num:                     %p\n",
              this_io_num->global_num);
 
-  bft_printf(_("\n"
-               "  pointer to local array:\n"
-               "    _global_num:                    %p\n"),
+  bft_printf("\n"
+             "  pointer to local array:\n"
+             "    _global_num:                    %p\n",
              this_io_num->global_num);
 
   if (this_io_num->global_num_size > 0) {
 
-    bft_printf(_("\n  global number:\n\n"));
+    bft_printf("\n  global number:\n\n");
     for (i = 0 ; i < this_io_num->global_num_size ; i++)
       bft_printf("  %10u : %10lu\n",
                  (unsigned)i + 1,
