@@ -22,7 +22,7 @@
   This file is part of the "Finite Volume Mesh" library, intended to provide
   finite volume mesh and associated fields I/O and manipulation services.
 
-  Copyright (C) 2004-2008  EDF
+  Copyright (C) 2004-2009  EDF
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -169,6 +169,51 @@ fvm_io_num_create_from_adj_s(const fvm_lnum_t  parent_entity_number[],
                              size_t            n_entities,
                              size_t            stride);
 
+/*----------------------------------------------------------------------------
+ * Creation of an I/O numbering structure based on an indexed adjacency.
+ *
+ * The corresponding entities do not need to be locally ordered.
+ *
+ * parameters:
+ *  parent_entity_number <-- pointer to list of selected entitie's parent's
+ *                           numbers, or NULL if all first n_ent entities
+ *                           are used
+ *  index                <-- index on entities for adjacency
+ *  adjacency            <-- entity adjacency (1 to n global numbering)
+ *  n_entities           <-- number of entities considered
+ *
+ * returns:
+ *  pointer to I/O numbering structure
+ *----------------------------------------------------------------------------*/
+
+fvm_io_num_t *
+fvm_io_num_create_from_adj_i(const fvm_lnum_t  parent_entity_number[],
+                             const fvm_lnum_t  index[],
+                             const fvm_gnum_t  adjacency[],
+                             fvm_lnum_t        n_entities);
+
+/*----------------------------------------------------------------------------
+ * Creation of an I/O numbering structure based on coordinates.
+ *
+ * The ordering is based on a Morton code, and it is expected that
+ * entities are unique (i.e. not duplicated on 2 or more ranks).
+ * In the case that 2 entities have a same Morton code, their global
+ * number will be determined by lexicographical ordering of coordinates.
+ *
+ * parameters:
+ *   coords     <-- pointer to entity coordinates (interlaced)
+ *   dim        <-- spatial dimension
+ *   n_entities <-- number of entities considered
+ *
+ * returns:
+ *  pointer to I/O numbering structure
+ *----------------------------------------------------------------------------*/
+
+fvm_io_num_t *
+fvm_io_num_create_from_coords(const fvm_coord_t  coords[],
+                              int                dim,
+                              size_t             n_entities);
+
 /*----------------------------------------------------------------------------
  * Destruction of a I/O numbering structure.
  *
